@@ -1,9 +1,12 @@
 """Query Builder"""
 
+from dataclasses import dataclass
 from functools import lru_cache
 from shlex import shlex
-from typing import Any, Iterable, Literal, Optional
+from typing import Any, Iterable, Literal, Optional, TypeAlias
+from uuid import uuid4
 
+from .subquery import SubQuery
 from .functions import ParsedFn, _function_extract
 from .errors import SecurityError
 from ._utils import check_one, null, check_iter, Null
@@ -12,14 +15,99 @@ from .locals import _SQLITETYPES, SQLACTION
 from .signature import Signature
 from .typings import _MasterQuery, Data, Orders
 
-ConditionDict = dict[str, Signature | ParsedFn | Any]
-ConditionList = list[tuple[str, Signature | ParsedFn]]
-Condition = ConditionDict | ConditionList | None
-CacheCond = tuple[tuple[str, Signature | ParsedFn], ...] | None
-CacheOrders = tuple[str, Literal["asc", "desc"]] | ParsedFn | None
-CacheData = tuple[str, ...]
+ConditionDict: TypeAlias = dict[str, Signature | ParsedFn | SubQuery | Any]
+ConditionList: TypeAlias = list[tuple[str, Signature | SubQuery | ParsedFn]]
+Condition: TypeAlias = ConditionDict | ConditionList | None
+CacheCond: TypeAlias = tuple[tuple[str, Signature | SubQuery | ParsedFn], ...] | None
+CacheOrders: TypeAlias = tuple[str, Literal["asc", "desc"]] | ParsedFn | None
+CacheData: TypeAlias = tuple[str, ...]
 OnlyColumn = tuple[str, ...] | str | ParsedFn
 DEFAULT_MAPPINGS = {value: value for value in _SQLITETYPES}
+SQL_ACTIONS = {"null": "set null"}
+MAX_SUBQUERY_STACK_LIMIT = 10
+
+NAMING_FORMAT = "{key}{suffix}__{call_id}_{depth}_{condition_id}"
+
+
+def set_subquery_stack_limit(value: int):
+    """Set subquery stack limit"""
+    global MAX_SUBQUERY_STACK_LIMIT  # pylint: disable=global-statement
+    if MAX_SUBQUERY_STACK_LIMIT <= 0:
+        raise ValueError("Cannot set limit below 1")
+    MAX_SUBQUERY_STACK_LIMIT = value
+
+
+def generate_ids():
+    """Generate ids for statements"""
+    return str(uuid4().int)
+
+
+class TableCreationExtractor:
+    """Base processor for table creation"""
+
+    def __init__(self, columns: Iterable[Column], type_mappings: dict[str, str] | None):
+        self.columns = columns
+        self.type_mappings = _get_type_mappings(type_mappings)
+        self.primaries: list[Column] = []
+        self.foreigns: list[Column] = []
+        self.string = ""
+
+    def process_columns(self):
+        """Process each columns"""
+        self.string = _iterate_etbc_step1(
+            self.columns, self.string, self.primaries, self.foreigns, self.type_mappings
+        )
+
+    def add_primary_keys(self):
+        """Add primary key constraints"""
+        if self.primaries:
+            self.string += (
+                f" primary key ({', '.join((col.name for col in self.primaries))}),"
+            )
+
+    def add_foreign_keys(self):
+        """Add foreign key constraints"""
+        for column in self.foreigns:
+            stable, sname = column.source, column.source_column
+            self.string += (
+                f" foreign key ({column.name}) references {stable} ({sname})"
+                f" on delete {_iterate_sql_action(column.on_delete)}"  # type: ignore
+                f" on update {_iterate_sql_action(column.on_update)},"  # type: ignore
+            )
+
+    def extract(self) -> str:
+        """Extract everything into a string"""
+        self.process_columns()
+        self.add_primary_keys()
+        self.add_foreign_keys()
+        return self.string[1:-1]
+
+
+@dataclass(frozen=True)
+class QueryParams:
+    """Encapsulates parameters for building SQL queries."""
+
+    table_name: str
+    condition: Optional[CacheCond] = None
+    only: Optional[OnlyColumn] = None
+    limit: int = 0
+    offset: int = 0
+    order: Optional[CacheOrders] = None
+    data: Optional[CacheData] = None
+
+    def __hash__(self):
+        """Custom hash function to ensure compatibility with lru_cache."""
+        return hash(
+            (
+                self.table_name,
+                self.condition,
+                self.only,
+                self.limit,
+                self.offset,
+                self.order,
+                self.data,
+            )
+        )
 
 
 def extract_table(  # pylint: disable=too-many-locals
@@ -85,49 +173,113 @@ def fetch_columns(_master_query: _MasterQuery):
     return extract_table(sql)
 
 
-def extract_signature(
-    filter_: Condition | CacheCond = None, suffix: str = "_check"  # type: ignore
+def _handle_subquery(key, value, depth):
+    subq, subq_data = extract_subquery(value.value, depth=depth + 1)
+    clause = f" {key} in ({subq})"
+    return clause, subq_data
+
+
+def _handle_in(key, middle, val, condition_id):
+    vals = tuple(
+        f":prop_{condition_id}_val_in{index}" for index, _ in enumerate(val.data)
+    )
+    clause = f" {key} {middle} ({', '.join(vals)})"
+    data = {key0[1:]: val0 for key0, val0 in zip(vals, val.data)}
+    return clause, data
+
+
+def _handle_between(key, middle, val):
+    vdata = val.data
+    if not all(isinstance(x, (int, float)) for x in vdata):
+        raise SecurityError("Values for between constraint is not int/float")
+    clause = f" {key} {middle} {vdata[0]!r} and {vdata[1]!r}"
+    return clause
+
+
+def _handle_like(key, middle, val):
+    vdata = val.data
+    clause = f" {key} {middle} {vdata!r}"
+    return clause
+
+
+def extract_signature(  # pylint: disable=too-many-locals
+    filter_: Condition | CacheCond = None, suffix: str = "_check", depth: int = 0
 ):
     """Extract filter signature."""
+    if depth < 0 or depth >= MAX_SUBQUERY_STACK_LIMIT:
+        raise RecursionError(
+            "Subquery builder has reached recursion limit of"
+            f"{MAX_SUBQUERY_STACK_LIMIT}"
+        )
     if filter_ is None:
         return "", {}
+
     if isinstance(filter_, (list, tuple)):
-        filter_: ConditionDict = dict(filter_)
-    string = "where"
+        filter_ = dict(filter_)
+
+    call_id = generate_ids()
+    clauses = []
     data: dict[str, Any] = {}
-    last = 1
+
     for key, value in filter_.items():
+        condition_id = generate_ids()
         if not isinstance(value, Signature):
             value = Signature(value, "=")
         old_data = value.value
+
         val = (
-            Signature(f":{key}{suffix}", value.generate(), value.data)
+            Signature(
+                ":"+NAMING_FORMAT.format(
+                    key=key,
+                    suffix=suffix,
+                    call_id=call_id,
+                    depth=depth,
+                    condition_id=condition_id,
+                ),
+                value.generate(),
+                value.data,
+            )
             if value.value is not null
             else value
         )
+
+        if isinstance(value.value, SubQuery):
+            clause, subq_data = _handle_subquery(key, value, depth)
+            clauses.append(clause)
+            data.update(subq_data)
+            continue
+
         middle = val.generate()
         if val.normal_operator:
-            string += f" {key}{middle}{val.value} and"
-            last = 4
-        if val.is_in:
-            vals = tuple(
-                (f":prop_val_in{index}" for index, _ in enumerate(val.data)))  # type: ignore
-            string += f" {key} {middle} ({', '.join(vals)}) "
-            for key0, val0 in zip(vals, val.data):  # type: ignore
-                data[key0[1:]] = val0
+            clauses.append(f" {key}{middle}{val.value}")
+        elif val.is_in:
+            clause, in_data = _handle_in(key, middle, val, condition_id)
+            clauses.append(clause)
+            data.update(in_data)
             continue
-        if val.is_between:
-            vdata: tuple[int, int] = val.data  # type: ignore
-            if not all(map(lambda x: isinstance(x, (int, float)), vdata)):
-                raise SecurityError("Values for between constraint is not int/float")
-            string += f" {key} {middle} {vdata[0]!r} and {vdata[1]!r} and"
-        if val.is_like:
-            vdata: str = val.data  # type: ignore
-            string += f" {key} {middle} {vdata!r} and"
-        if val.value is not null:
-            data[f"{key}{suffix}"] = old_data
+        elif val.is_between:
+            clause = _handle_between(key, middle, val)
+            clauses.append(clause)
+        elif val.is_like:
+            clause = _handle_like(key, middle, val)
+            clauses.append(clause)
 
-    return string[:-last], data
+        if val.value is not null:
+            data[
+                NAMING_FORMAT.format(
+                    key=key,
+                    suffix=suffix,
+                    call_id=call_id,
+                    depth=depth,
+                    condition_id=condition_id,
+                )
+            ] = old_data
+
+    if not clauses:
+        return "", data
+
+    where_clause = "where" + " and".join(clauses)
+    return where_clause, data
 
 
 def basic_extract(table_creation: str):  # pylint: disable=too-many-locals
@@ -286,6 +438,18 @@ def extract_single_column(column: Column):
     return string
 
 
+def _process_column_constraints(column: Column, ctype: str) -> str:
+    """Process constraints for a single column."""
+    constraints = f" {column.name} {ctype}"
+    if not column.nullable:
+        constraints += " not null"
+    if column.unique:
+        constraints += " unique"
+    if column.default:
+        constraints += f" default {repr(column.default)}"
+    return constraints
+
+
 def _iterate_etbc_step1(
     columns: Iterable[Column],
     string: str,
@@ -295,49 +459,48 @@ def _iterate_etbc_step1(
 ):
     for column in columns:
         ctype = maps.get(column.type, column.type)
-        string += f" {column.name} {ctype}"
+        string += _process_column_constraints(column, ctype)
         if column.raw_source:
             foreigns.append(column)
         if column.primary:
             primaries.append(column)
-        if not column.nullable:
-            string += " not null"
-        if column.unique:
-            string += " unique"
-        if column.default:
-            string += f" default {repr(column.default)}"
         string += ","
     return string
 
 
 def _iterate_sql_action(action: SQLACTION):
-    if action == "null":
-        return "set null"
-    return action
+    return SQL_ACTIONS.get(action, action)
+
+
+def _get_type_mappings(type_mappings: dict[str, str] | None) -> dict[str, str]:
+    """Get the merged type mappings."""
+    maps = DEFAULT_MAPPINGS.copy()
+    if type_mappings:
+        maps.update(type_mappings)
+    return maps
 
 
 def extract_table_creations(
     columns: Iterable[Column], type_mappings: dict[str, str] | None = None
 ):
     """Extract columns classes to sqlite table creation query."""
-    maps: dict[str, str] = DEFAULT_MAPPINGS.copy()
-    if type_mappings:
-        maps.update(type_mappings)
-    primaries: list[Column] = []
-    foreigns: list[Column] = []
-    string = _iterate_etbc_step1(columns, "", primaries, foreigns, maps)
+    extractor = TableCreationExtractor(columns, type_mappings)
+    return extractor.extract()
 
-    if primaries:
-        string += f" primary key ({', '.join((col.name for col in primaries))}),"
-    if not foreigns:
-        return string[1:-1]
-    for column in foreigns:
-        stable, sname = column.source, column.source_column
 
-        string += f" foreign key ({column.name}) references {stable} ({sname})\
- on delete {_iterate_sql_action(column.on_delete)} on update {_iterate_sql_action(column.on_update)},"  # type: ignore # pylint: disable=line-too-long
-        # ! This might be a buggy code, i'm not sure yet.
-    return string[1:-1]
+@lru_cache
+def extract_subquery(subquery: SubQuery, depth: int = 1):
+    """Extract subquery into a valid SQL statement"""
+    return _build_select(
+        QueryParams(
+            subquery.table,
+            subquery.where,
+            subquery.cols,
+            subquery.limit,
+            subquery.orders,  # type: ignore
+        ),
+        depth=depth,
+    )
 
 
 def _select_onlyparam_parse(data: str | ParsedFn):
@@ -384,66 +547,62 @@ def _parse_orders(order: CacheOrders):
         return ", ".join(f"{ord_} {order_by}" for ord_, order_by in order)
     raise TypeError("What?", type(order))
 
+
 def _remove_null(condition: dict[str, Any]) -> dict[str, Any]:
     new = condition.copy()
     for key, value in condition.items():
         if value is Null:
             del new[key]
     if not new:
-        raise ValueError("After removing Null sentinel value, new data that would be inserted"
-                         "/updated returns empty dictionary.")
+        raise ValueError(
+            "After removing Null sentinel value, new data that would be inserted"
+            "/updated returns empty dictionary."
+        )
     return new
 
-@lru_cache
-def _build_select(  # pylint: disable=too-many-arguments
-    table_name: str,
-    condition: CacheCond,
-    only: OnlyColumn = None,  # type: ignore
-    limit: int = 0,
-    offset: int = 0,
-    order: CacheOrders = None,
-):
-    check_one(table_name)
-    cond, data = extract_signature(condition)
-    check_iter(only or ())  # type: ignore
-    only_ = "*"
-    if only and isinstance(only, ParsedFn):
-        only_, _ = only.parse_sql()
-    elif isinstance(only, tuple):
-        only_ = f"{', '.join(column_name for column_name in only)}"
-    elif only != "*" and isinstance(only, str):
-        only_ = check_one(only)  # type: ignore
 
-    query = f"select {only_} from {table_name}"
+@lru_cache
+def _build_select(query_params: QueryParams, depth: int = 0):
+    if depth < 0 or depth >= MAX_SUBQUERY_STACK_LIMIT:
+        raise RecursionError(
+            "Subquery builder has reached recursion limit of"
+            f"{MAX_SUBQUERY_STACK_LIMIT}"
+        )
+    check_one(query_params.table_name)
+    cond, data = extract_signature(query_params.condition, depth=depth)
+    check_iter(query_params.only or ())  # type: ignore
+    only_ = "*"
+    if query_params.only and isinstance(query_params.only, ParsedFn):
+        only_, _ = query_params.only.parse_sql()
+    elif isinstance(query_params.only, tuple):
+        only_ = f"{', '.join(column_name for column_name in query_params.only)}"
+    elif query_params.only != "*" and isinstance(query_params.only, str):
+        only_ = check_one(query_params.only)  # type: ignore
+
+    query = f"select {only_} from {query_params.table_name}"
     if cond:
         query += f" {cond}"
-    if order and isinstance(order, tuple):
-        query += f" order by {_parse_orders(order)}"
-    if limit:
-        query += f" limit {limit}"
-    if offset:
-        query += f" offset {offset}"
+    if query_params.order and isinstance(query_params.order, tuple):
+        query += f" order by {_parse_orders(query_params.order)}"
+    if query_params.limit:
+        query += f" limit {query_params.limit}"
+    if query_params.offset:
+        query += f" offset {query_params.offset}"
 
     return query, data
 
 
 @lru_cache
-def _build_update(
-    table_name: str,
-    new_data: CacheData,
-    condition: CacheCond = None,
-    limit: int = 0,
-    order: CacheOrders = None,
-):
-    check_one(table_name)
-    cond, data = extract_signature(condition)
-    new_str, updated = build_update_data(new_data)
-    query = f"update {table_name} set {new_str} {cond}"
-    if limit:
+def _build_update(query_params: QueryParams):
+    check_one(query_params.table_name)
+    cond, data = extract_signature(query_params.condition)
+    new_str, updated = build_update_data(query_params.data)  # type: ignore
+    query = f"update {query_params.table_name} set {new_str} {cond}"
+    if query_params.limit:
         query = query.replace(cond, "")
-        query += _setup_limit_patch(table_name, cond, limit)
-    if order:
-        query += f" order by {_parse_orders(order)}"
+        query += _setup_limit_patch(query_params.table_name, cond, query_params.limit)
+    if query_params.order:
+        query += f" order by {_parse_orders(query_params.order)}"
     # ? Require manual intervention to make sure updated is sync as
     # print(query)
     return query, data, updated
@@ -452,20 +611,15 @@ def _build_update(
 
 
 @lru_cache
-def _build_delete(
-    table_name: str,
-    condition: CacheCond = None,
-    limit: int = 0,
-    order: CacheOrders = None,
-):
-    check_one(table_name)
-    cond, data = extract_signature(condition)
-    query = f"delete from {table_name} {cond}"
-    if limit:
+def _build_delete(query_params: QueryParams):
+    check_one(query_params.table_name)
+    cond, data = extract_signature(query_params.condition)
+    query = f"delete from {query_params.table_name} {cond}"
+    if query_params.limit:
         query = query.replace(cond, "")
-        query += _setup_limit_patch(table_name, cond, limit)
-    if order:
-        query += f" order by {_parse_orders(order)}"
+        query += _setup_limit_patch(query_params.table_name, cond, query_params.limit)
+    if query_params.order:
+        query += f" order by {_parse_orders(query_params.order)}"
     return query, data
 
 
@@ -500,7 +654,15 @@ def build_select(  # pylint: disable=too-many-arguments
         tuple[str, dict[str, Any]]: query and query data
     """
     cond, order_, _ = _setup_hashable(condition, order)
-    return _build_select(table_name, cond, only, limit, offset, order_)
+    params = QueryParams(
+        table_name=table_name,
+        condition=cond,
+        only=only,
+        limit=limit,
+        offset=offset,
+        order=order_,  # type: ignore
+    )
+    return _build_select(params)
 
 
 def build_update(
@@ -524,7 +686,14 @@ def build_update(
     """
     new_data = _remove_null(new_data)
     cond, order_, ndata = _setup_hashable(condition, order, new_data)
-    query, check, updated = _build_update(table_name, ndata, cond, limit, order_)
+    params = QueryParams(
+        table_name=table_name,
+        condition=cond,
+        limit=limit,
+        order=order_,  # type: ignore
+        data=ndata,
+    )
+    query, check, updated = _build_update(params)
     return query, check | combine_keyvals(updated, new_data)
 
 
@@ -547,7 +716,10 @@ def build_delete(
     """
 
     cond, order_, _ = _setup_hashable(condition, order)
-    return _build_delete(table_name, cond, limit, order_)
+    params = QueryParams(
+        table_name, condition=cond, limit=limit, order=order_  # type: ignore
+    )
+    return _build_delete(params)
 
 
 def build_insert(table_name: str, data: Data) -> tuple[str, dict[str, Any]]:
@@ -580,4 +752,5 @@ __all__ = [
     "combine_keyvals",
     "extract_single_column",
     "extract_table_creations",
+    "set_subquery_stack_limit",
 ]
